package com.autentia.tutoriales.modelo.entidades;

import java.util.List;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.JoinColumn;
import javax.persistence.JoinTable;
import javax.persistence.ManyToMany;
import javax.persistence.ManyToOne;

/**
 * Representa una entidad de tipo Libro.
 * @author fjmpaez
 *
 */
@Entity
public class Libro extends TransferObject {
	
	private String titulo;
	private String isbn;	
	private List<Autor> autores;	
	private Categoria categoria;
	private Socio socio;
	
	@Id
	@GeneratedValue(strategy = GenerationType.AUTO)
	public Integer getId() {
		return id;
	}	

	public String getTitulo() {
		return titulo;
	}

	public void setTitulo(String titulo) {
		this.titulo = titulo;
	}

	public String getIsbn() {
		return isbn;
	}

	public void setIsbn(String isbn) {
		this.isbn = isbn;
	}
	
	/**
	 * Modificamos la forma de obtener los autores del libro,
	 * ya que al mostrarlos en el listado de libros, es necesario
	 * obtenerlos siempre para evitar un error.
	 * @return
	 */
	@ManyToMany(fetch=FetchType.EAGER)
	@JoinTable(name="Libro_Autor",
			joinColumns=
				@JoinColumn(name="libro",referencedColumnName="id"),
			inverseJoinColumns=
				@JoinColumn(name="autor",referencedColumnName="id")
			  )
	public List<Autor> getAutores() {
		return autores;
	}

	public void setAutores(List<Autor> autores) {
		this.autores = autores;
	}
	
	@ManyToOne
	@JoinColumn(name = "categoriaId",referencedColumnName="id")
	public Categoria getCategoria() {
		return categoria;
	}

	public void setCategoria(Categoria categoria) {
		this.categoria = categoria;
	}
	
	@ManyToOne
	@JoinColumn(name="socioId",referencedColumnName="id")
	public Socio getSocio() {
		return socio;
	}

	public void setSocio(Socio socio) {
		this.socio = socio;
	}

	public String toString() {
		return "id="+id+",titulo="+this.titulo+",isbn="+this.isbn+",categoria="+this.getCategoria().getNombre();
	}
	
}
