package com.autentia.tutoriales.modelo.services;

import java.util.List;
import javax.ejb.Stateless;
import javax.persistence.EntityManager;
import javax.persistence.NoResultException;
import javax.persistence.PersistenceContext;
import javax.persistence.Query;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import com.autentia.tutoriales.modelo.entidades.TransferObject;

/**
 * Bean que implementa el interfaz Local Dao.
 * @author fjmpaez
 *
 */
@Stateless
public class DaoImpl implements Dao {

	private static final Log log = LogFactory.getLog(DaoImpl.class);
	@PersistenceContext
	protected EntityManager em;
	@SuppressWarnings("unchecked")
	public <T extends TransferObject> List<T> findAll(Class<T> transferObjectClass, String sortColumn, boolean ascending) {
		log.debug("DaoImpl:findAll");
		final String entityName = transferObjectClass.getSimpleName();
		final Query query;

		if (ascending) {
			query = em.createQuery("from " + entityName + " e order by e." + sortColumn + " asc");
		} else {
			query = em.createQuery("from " + entityName + " e order by e." + sortColumn + " desc");
		}

		final List<T> resultList = query.getResultList();
		if (log.isTraceEnabled()) {
			log.trace(resultList.size() + " " + entityName + " recovered from database.");
		}
		return resultList;
	}

	public <T extends TransferObject> T find(Class<T> transferObjectClass, Integer id) {
		return em.find(transferObjectClass, id);
	}
		
	private Query createQuery(String query, FilterParameter ... params) {		
		Query qQuery = em.createQuery(query);
		if (params != null && params.length>0) {
			for(FilterParameter fP:params) {			
				qQuery.setParameter(fP.getField(), fP.getValue());
			}			
		}
		return qQuery;
	}
		
	@SuppressWarnings("unchecked")
	public <T extends TransferObject> List<T> findAllByQuery(String query, FilterParameter ... params) {
		Query qQuery = createQuery(query, params);
		log.debug("CONSULTA:" + qQuery.toString());
		return qQuery.getResultList();
	}
	
	@SuppressWarnings("unchecked")
	public <T extends TransferObject> T findSingleByQuery(String query, FilterParameter ... params) {
		Query qQuery = createQuery(query, params);
		log.debug("CONSULTA:" + qQuery.toString());
		Object obj = null;
		try {
			obj = qQuery.getSingleResult();
		} catch (NoResultException e) {
			log.debug("NO se han encontrado resultados", e);
		}
		return (T)obj;
	}
	
	public <T extends TransferObject> List<T> findAllAndFilterLike(
			Class<T> transferObjectClass, String sortColumn, boolean ascending, boolean or, FilterParameter ... params) {
		log.debug("DaoImpl:findAllAndFilterLike");
		final String entityName = transferObjectClass.getSimpleName();		
		StringBuffer sbQuery = new StringBuffer("from ").append(entityName).append(" e");	
		final Query query;		
		
		FilterParameter [] params2 = null;		
				
		
		boolean first=true;		
		int i = 0;
		if(params!=null && params.length>0) {
			
			params2 = new FilterParameter[params.length];
			
			for(FilterParameter fP:params) {
				if(first) {
					sbQuery.append(" where e.").append(fP.getField()).append(" like :").append("p"+i);
				} else { 
					sbQuery.append(or?" or":" and").append(" e.").append(fP.getField()).append(" like :").append("p"+i);							
				}				
				
				first=false;				
				
				FilterParameter fPAux = new FilterParameter("p"+i,fP.getValue()+"%");
				params2[i] = fPAux;
				i++;
			}			
		}		
		
		if (ascending) 
			sbQuery.append(" order by e." + sortColumn + " asc");			
		else 
			sbQuery.append(" order by e." + sortColumn + " desc");			
		
		query = createQuery(sbQuery.toString(), params2);		
				
		final List<T> resultList = query.getResultList();
		if (log.isTraceEnabled()) {
			log.trace(resultList.size() + " " + entityName + " recovered from database.");
		}
		return resultList;		
	}	
}
