package com.autentia.tutoriales.modelo.util;

import javax.naming.InitialContext;
import javax.naming.NamingException;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

/**
 * Clase localizadora de EJBs a usar desde la capa WEB.
 * @author fjmpaez
 *
 */
public class EjbLocator {
	
	private static Log log = LogFactory.getLog(EjbLocator.class);	
	private static final InitialContext ctx;

	static {
		try {
			ctx = new InitialContext();
		} catch (NamingException e) {
			log.fatal("It is not possible to create a new InitialContext.", e);
			throw new RuntimeException(e);
		}
	}
	
	/**
	 * Constructor privado.
	 */
	private EjbLocator() {		
	}
		
	/**
	 * Retorna una referencia a un EJB Local
	 */
	@SuppressWarnings("unchecked")
	public static <T> T lookupLocalBean(Class<? extends T> beanClass) throws NamingException {
		final String jndiName = beanClass.getSimpleName() + "/local";
		final T bean = (T)jndiLookup(jndiName);
		return bean;
	}

	private static Object jndiLookup(String jndiName) throws NamingException {
		final Object obj;
		try {
			obj = ctx.lookup(jndiName);
			if (log.isDebugEnabled()) log.debug("Found JNDI name: " + jndiName);
		} catch (NamingException e) {
			log.fatal("Can not found JNDI name: " + jndiName);
			throw e;
		}
		return obj;
	}	
	
}
